#/bin/bash
# This script was indeed self-updated!

# Catch signals so we don't leave zombie tasks behind if we Ctrl^C
# No idea why but it seems to prevent the script from working? Disable!
#trap "exit" INT TERM ERR
#trap "kill 0" EXIT

BASEDIR="$HOME/.git-build"

# So scripts can know we're still running (for autoupdater)
touch $BASEDIR/.LOCK

for project in $BASEDIR/*.source; do
	p_name="$(basename $project .source)"
	echo "[$p_name] START"
	[ -f $BASEDIR/$p_name.branch ] && p_branch="$(cat $BASEDIR/$p_name.branch)" || p_branch="master"
	p_dir="$BASEDIR/.$p_name"
	if [ ! -d $p_dir ]; then
		source="$(cat $BASEDIR/$p_name.source)"
		echo "[$p_name] CLONE $source"
		# Don't forget the git submodules!
		git clone --recursive "$source" "$p_dir"
		[[ $? != 0 ]] && echo "[$p_name] CLONE FAILED" && exit 1
		if [[ "$p_branch" != "master" ]]; then
			cd $p_dir
			echo "[$p_name] CHECKOUT BRANCH $p_branch"
			git checkout "$p_branch"
			[[ $? != 0 ]] && echo "[$p_name] CHECKOUT FAILED" && exit 1
		fi
	fi
	cd "$p_dir"
	# Refresh remote before comparing with local
	git fetch --quiet origin
	git diff --quiet remotes/origin/$p_branch
	if [[ $? != 0 ]]; then
		echo "[$p_name] PULL"
		# Update all submodules, for now only when the main repo changed (TODO)
		git pull --quiet --recurse-submodules
		[[ $? != 0 ]] && echo "[$p_name] PULL FAILED" && exit 1
		echo "[$p_name] RUN"
	fi
		# Run in background and redirect output to $p_name.log
		(GITBUILDDIR="$BASEDIR" nohup $BASEDIR/$p_name $p_name > $BASEDIR/$p_name.log 2> $BASEDIR/$p_name.err) &
done

rm $BASEDIR/.LOCK
